/*
 * Copyright (C) 2021  Topview Optronics Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef __LENS_CONTROL_H__
#define __LENS_CONTROL_H__

#define LENS_IOC_MAGIC     'L'
#define MAX_RAW_CMD 20
typedef enum
{
    CODE_DATA_TARGET_C = 0,
    CODE_DATA_TARGET_DSP,
    CODE_DATA_TARGET_DSP_DATA
}E_CODE_DATA_TARGET;

struct lens_steps {
    /* position: current zoom steps */
    /* moving: - zoom in, + zoom out */
    int zoom;
    /* position: current focus steps */
    /* moving: - focus near, + focus far */
    int focus;
};

struct lens_pid_params {
    unsigned long xkp;
    unsigned long xki;
    unsigned long xkd;
    unsigned short x9b;
    unsigned long x_tau;
    unsigned long x_k1;
    unsigned long out_average;
    unsigned long hall_average;
    unsigned long dummy_counter;
};

struct lens_pi_gpio {
    int zoom;
    int focus;
};

struct piris_setup_data {
    unsigned char phaseCount;
    char phaseTable[8][4];
    unsigned int sizeOfTable;
};

struct code_data
{
    E_CODE_DATA_TARGET Target;
    unsigned int StartAddr;
    unsigned int EndAddr;
    unsigned int CheckSumHigh;
    unsigned int CheckSumLow;
    unsigned int Size;
    const unsigned char* pData;
};

typedef struct msio_cmd
{
    unsigned char txd[3];
    unsigned char rxd[3];
} __attribute__((__packed__)) msio_cmd;

typedef struct msio_n_cmd
{
    int number; //number of command to be written
    int wr_cnt; //how many commmand is acctaully written
    msio_cmd cmd[MAX_RAW_CMD];
} __attribute__((__packed__)) msio_n_cmd;


#define LENS_IOC_STOP_MOVE _IOW(LENS_IOC_MAGIC, 3, int)
#define LENS_IOC_PIRIS_OPEN _IOW(LENS_IOC_MAGIC, 4, int)
#define LENS_IOC_PIRIS_CLOSE _IOW(LENS_IOC_MAGIC, 5, int)
#define LENS_IOC_FOCUS_MOVE_SPEED _IOW(LENS_IOC_MAGIC, 6, int)
#define LENS_IOC_ZOOM_MOVE_SPEED _IOW(LENS_IOC_MAGIC, 7, int)
#define LENS_IOC_GET_BUTTON _IOR(LENS_IOC_MAGIC, 8, int)
#define LENS_IOC_PIRIS_SETUP _IOW(LENS_IOC_MAGIC, 9, struct piris_setup_data)
#define LENS_IOC_DCIRIS_SETUP _IO(LENS_IOC_MAGIC, 10)
#define LENS_IOC_RELATIVE_FOCUS_MOVE _IOW(LENS_IOC_MAGIC, 20, int)
#define LENS_IOC_RELATIVE_ZOOM_MOVE _IOW(LENS_IOC_MAGIC, 21, int)
#define LENS_IOC_GET_FOCUS_STATUS _IOR(LENS_IOC_MAGIC, 22, int)
#define LENS_IOC_GET_ZOOM_STATUS _IOR(LENS_IOC_MAGIC, 23, int)
#define LENS_IOC_SET_ZOOM_VOLTAGE _IOW(LENS_IOC_MAGIC, 35, int)
#define LENS_IOC_SET_FOCUS_VOLTAGE _IOW(LENS_IOC_MAGIC, 36, int)
#define LENS_IOC_SET_PIRIS_VOLTAGE _IOW(LENS_IOC_MAGIC, 37, int)
#define LENS_IOC_SET_PI _IOW(LENS_IOC_MAGIC, 52, int)

#define DIRECTION_ZOOM_OUT -1
#define DIRECTION_ZOOM_IN  1
#define DIRECTION_FOCUS_NEAR 1
#define DIRECTION_FOCUS_FAR  -1

#define ZOOM_PI_MASK 0x1
#define FOCUS_PI_MASK 0x2

#define TYPE_ZOOM 0
#define TYPE_FOCUS 1

#define MAX_CSIO_BOOT_DATA_CNT 4
#define MAX_CSIO_CMD_DATA_CNT 5
#define MAX_CSIO_CMD_READ 6
#define MAX_MSIO_DATA_CNT 3

#endif /* __LENS_CONTROL_H__ */
