/*
 * Copyright (C) 2021  Topview Optronics Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <linux/module.h>
#include <linux/gpio.h>
#include <linux/of_gpio.h>
#include "lens_gpio.h"

void setup_gpio_direction(int gpio_id, u32 direction) {
	char gpio_label[16] = {0};

	if (!gpio_is_valid(gpio_id)) {
		printk("Lens_GPIO: gpio %d is not valid", gpio_id);
	} else {
		snprintf(gpio_label, sizeof(gpio_label), "gpio %d\n", gpio_id);
		if (gpio_request(gpio_id, gpio_label) >= 0) {
			switch (direction) {
				case GPIO_OUTPUT:
					gpiod_direction_output(gpio_to_desc(gpio_id), 0);
					break;
				case GPIO_INPUT:
					gpiod_direction_input(gpio_to_desc(gpio_id));
					break;
				default:
					break;
			}
		} else {
			printk("Lens_GPIO: Can not request gpio %d", gpio_id);
		}
	}
}

u32 read_gpio(int gpio_id) {
	return gpiod_get_value_cansleep(gpio_to_desc(gpio_id));
}

void set_gpio(int gpio_id) {
	gpiod_set_value_cansleep(gpio_to_desc(gpio_id), 1);
}

void clear_gpio(int gpio_id) {
	gpiod_set_value_cansleep(gpio_to_desc(gpio_id), 0);

}

void setup_gpio_interrupt(int gpio_id,
		u32 trigger_mode, irq_handler_t thread_fn) {
	unsigned long irqflags = IRQF_TRIGGER_NONE;
	int irq, err;


	switch (trigger_mode) {
		case TRIGGER_DUAL_EDGE:
			irqflags = IRQF_TRIGGER_FALLING | IRQF_TRIGGER_RISING;
			break;
		case TRIGGER_UP_EDGE:
			irqflags = IRQF_TRIGGER_RISING;
			break;
		case TRIGGER_DOWN_EDGE:
			irqflags = IRQF_TRIGGER_FALLING;
			break;
		case TRIGGER_HIGH_LEVEL:
			irqflags = IRQF_TRIGGER_HIGH;
			break;
		case TRIGGER_LOW_LEVEL:
			irqflags = IRQF_TRIGGER_LOW;
			break;
		default:
			return;
	}

	irq = gpiod_to_irq(gpio_to_desc(gpio_id));

	if (irq >= 0) {
		err = request_threaded_irq(irq, NULL, thread_fn, irqflags | IRQF_ONESHOT, "motor_lens", NULL);
	}
}

void free_gpio(int gpio_id) {
	gpio_free(gpio_id);
}

void enable_gpio_interrupt(int gpio_id) {
}

void disable_gpio_interrupt(int gpio_id) {
	int irq;

	irq = gpiod_to_irq(gpio_to_desc(gpio_id));

	free_irq(irq, NULL);
}

