/*
 * Copyright (C) 2021  Topview Optronics Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef __LENS_MOTOR_H__
#define __LENS_MOTOR_H__

struct stepping_motor {
  void (*set_speed)(u32 speed);
  void (*set_voltage)(u32 voltage);
  u32 (*get_voltage)(void);
  void (*forward)(u32 steps);
  void (*reverse)(u32 steps);
  void (*stop)(void);
};

enum daynight {
  DAY_MODE,
  NIGHT_MODE
};

struct dn_switch {
  void (*set)(enum daynight daynight);
};

struct pi_switch {
  void (*set)(int bitmask);
};

struct piris_op {
  void (*open)(u32 steps);
  void (*close)(u32 steps);
  void (*setup)(int phasecount, char *phasetable);
  void (*set_voltage)(u32 voltage);
  u32 (*get_voltage)(void);
};

struct motor_driver {
  s32 (*initialize)(void *ptr);
  void (*release)(void);
  struct stepping_motor stepping[2];
  struct dn_switch daynight;
  struct pi_switch pi;
  struct piris_op piris;
};

enum motor_driver_type {
  MOTOR_DRIVER_PT5111,
  MOTOR_DRIVER_R2J30516,
  MOTOR_DRIVER_COUNT
};

s32 open_motor_driver(void *ptr, struct motor_driver* motor, u32 type);

#define EXT_GPIO_BASE 496

#define EXPAND_GPIO( x ) ( EXT_GPIO_BASE + x )

/* P-IRIS */
//#define PIRIS_IN3          93
//#define PIRIS_IN4         142
//#define PIRIS_IN5          78
//#define PIRIS_IN6         117

#define PIRIS_POWER_ON          0xCE00  //11 0011 1 0 00 000000 (PT5111 page#34)
#define PIRIS_POWER_OFF         0xCC00  //11 0011 0 0 00 000000 (PT5111 page#34)

#endif /* __LENS_MOTOR_H__ */
