/*
 * Copyright (C) 2021  Topview Optronics Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
#ifndef __PT5111_DEFINE_H__
#define __PT5111_DEFINE_H__

/* Channel selection on the motor driver */
#define CHANNEL12                   0x0000      /* Channels 1 and 2 */
#define CHANNEL34                   0x4000      /* Channels 3 and 4 */
#define CHANNEL57                   0x8000      /* Channels 5 and 7 */
#define COMMON                      0xC000      /* Common items */

//////////////////////////////////////////////////////////////////////////////////////////////////////

/* Register selection for channels 1~7 */
#define DIRECTION                   0x0000      /* Register 0 */   /* Direction of rotation, driving mode, acceleration/deceleration, and initial position set by excitation */
#define NO_PULSES1                  0x0400      /* Register 1 */   /* Number of pulses 1 */
#define NO_PULSES2                  0x0800      /* Register 2 */   /* Number of pulses 2 */
#define PULSE_RATE0                 0x0C00      /* Register 3 */   /* Pulse rate 0 */
#define PULSE_RATE_RANGE            0x1000      /* Register 4 */   /* Pulse-rate range and number of steps */
#define NO_PULSES_ACC_DEC           0x1400      /* Register 5 */   /* Number of pulses per stage in acceleration and deceleration */
#define PULSE_RATE1                 0x1800      /* Register 6 */   /* Pulse rate 1 in acceleration and deceleration */
#define PULSE_RATE2                 0x1C00      /* Register 7 */   /* Pulse rate 2 in acceleration and deceleration */
#define PULSE_RATE3                 0x2000      /* Register 8 */   /* Pulse rate 3 in acceleration and deceleration */
#define PULSE_RATE4                 0x2400      /* Register 9 */   /* Pulse rate 4 in acceleration and deceleration */
#define PRE_EXCIT                   0x2800      /* Register A */   /* Pre-pulse-generation excitation time */
#define POST_EXCIT                  0x2C00      /* Register B */   /* Post-pulse-generation excitation time */
#define PWM57_SETTING               0x3000      /* Register C */   /* CH5/7 PWM setting */
#define VOLTAGE                     0x3400      /* Register D */   /* Voltage amplitude, Gain */
#define PWM5_DUTY                   0x3400      /* Register D */   /* duty cycle of ch5 */
#define READ_PULSES_POS             0x3800      /* Register E */   /* Read of pulses position */
#define PWM7_DUTY                   0x3800      /* Register E */   /* duty cycle of ch7 */
#define OPERATION_CTRL              0x3C00      /* Register F */   /* Per-channel operation-enabling control */

/* Register selection for common items */
#define EXT_OUT                     0x0400      /* Register 1 */   /* EXT and MOB output */
#define PI_CTRL                     0x0800      /* Register 2 */   /* Control of PI OUTput value */
#define PS_CTRL                     0x0C00      /* Register 3 */   /* Power supply control */
#define INPUT_CTRL                  0x1400      /* Register 5 */   /* Control of input pins */
#define CHAN5_CTRL                  0x1800      /* Register 6 */   /* Channel 5 control mode and setting of constant current value */
#define CHAN6_CTRL                  0x1C00      /* Register 7 */   /* Channel 6 control mode, setting of constant current value, weak excitation ON/OFF, and setting of weak excitation current value */
#define SW_RST                      0x2000      /* Register 8 */   /* Software reset and deceleration to a stop */
#define READ_STATUS                 0x2800      /* Register A */   /* Read of status */
#define COMM_OP_CTRL                0x3C00      /* Register F */   /* Operation-enabling control */

//////////////////////////////////////////////////////////////////////////////////////////////////////

/* Register value for channels 1~7 */
/* Register 0 */
#define FORWARD                     0x0000
#define REVERSE                     0x0200

#define MICRO_MODE                  0x0000      /* Micro-stepping */
#define MODE_1_2_100                0x0040      /* 1-2 phase 100% */
#define MODE_1_2_70                 0x0080      /* 1-2 phase 70% */
#define MODE_2_2                    0x00C0      /* 2-2 phase*/

#define NO_ACC_NO_DEC               0x0000      /* No acceleration/no deceleration control */
#define NO_ACC_DEC                  0x0010      /* No acceleration control/with deceleration control */
#define ACC_NO_DEC                  0x0020      /* With acceleration control/no deceleration control */
#define ACC_DEC                     0x0030      /* Acceleration/deceleration control */

#define INIT_POS0                   0x0000
#define INIT_POS1                   0x0002
#define INIT_POS2                   0x0004
#define INIT_POS3                   0x0006
#define INIT_POS4                   0x0008
#define INIT_POS5                   0x000A
#define INIT_POS6                   0x000C
#define INIT_POS7                   0x000E
#define INIT_POS_EN                 0x0001

#define CH5_FS                      0x0100      /* Driving mode: Channels 5 and 7 are full-swing PWM */
#define CH5_FS_PARALLEL             0x0140      /* Driving mode: Channel 5 and 7 are full-swing PWM (parallel driver by tow channels) */
#define CH5_CC                      0x0180      /* Driving mode: constant current is driven on channel 5, channel 7 is full-swing PWM*2 */

/* Register 1 and 2*/
/* number of pulses for motor control = (number of pulses 2) << 10 + (number of pulses 1) */
/* range from 0 to 1048575 */

/* Register 4 */
#define PRR_02u                     0x0280      /* set pulse rate range 0.2u sec */
#define PRR_04u                     0x0240      /* set pulse rate range 0.4u sec */
#define PRR_08u                     0x0200      /* set pulse rate range 0.8u sec */
#define PRR_16u                     0x01C0      /* set pulse rate range 1.6u sec */
#define PRR_32u                     0x0180      /* set pulse rate range 3.2u sec */
#define PRR_64u                     0x0140      /* set pulse rate range 6.4u sec */
#define PRR_128u                    0x0100      /* set pulse rate range 12.8u sec */
#define PRR_256u                    0x00C0      /* set pulse rate range 25.6u sec */
#define PRR_512u                    0x0080      /* set pulse rate range 51.2u sec */
#define PRR_1024u                   0x0040      /* set pulse rate range 102.4u sec */
#define PRR_2048u                   0x0000      /* set pulse rate range 204.8u sec */

/* set the number of steps when micro-stepping has been selected as drive mode */
#define MICRO32                     0x0030      /* 32 steps */
#define MICRO64                     0x0020      /* 64 steps */
#define MICRO128                    0x0010      /* 128 steps */
#define MICRO256                    0x0000      /* 256 steps */

/* Register A */
#define EXCITIME_0                  0x0000      /* 0ms */
#define EXCITIME_1                  0x0010      /* 0.8ms */
#define EXCITIME_2                  0x0020      /* 1.6ms */
#define EXCITIME_3                  0x0030      /* 2.5ms */
#define EXCITIME_4                  0x0040      /* 3.3ms */
#define EXCITIME_5                  0x0050      /* 4.0ms */
#define EXCITIME_6                  0x0060      /* 5.0ms */
#define EXCITIME_7                  0x0070      /* 5.7ms */
#define EXCITIME_8                  0x0080      /* 6.5ms */
#define EXCITIME_CONTINUE           0x03F0      /* continuous */

/* Register C */
/* PWM direction */
#define PWM5_FORWARD                0x0000
#define PWM5_REVERSE                0x0200
/* PWM stop mode */
#define PWM5_STOP_OFF               0x0000
#define PWM5_STOP_BRAKE             0x0100
/* PWM Driver Mode for CH5 */
#define PWM5_BRAKE_MODE             0x0000
#define PWM5_REVERSE_MODE           0x0080
/* PWM freq for CH5 */
#define PWM5_FREQ1                  0x0000
#define PWM5_FREQ2                  0x0040

/* Register D */
#define VOLT12_32                   0x0240
#define VOLT12_34                   0x0200
#define VOLT34_32                   0x0240
#define VOLT34_34                   0x0200
#define CARRFEQ0                    0x0000      /* 230khz */
#define CARRFEQ1                    0x0020      /* 400khz */

/* Register F */
#define ACTIVE                      0x0200      /* excitation is on, and pulse transitions proceed */
#define STOP                        0x0000
#define HOLD_ON                     0x0100      /* excitation is on, but pulse transitions stop */
#define HOLD_OFF                    0x0000
#define EXCIT_ON                    0x0080      /* excitation is on */
#define EXCIT_OFF                   0x0000


/* For COMM chanel */
/* Register 1 EXT output */
#define EXT_EXCIT                   0x0000
#define EXT_PULSE                   0x0100
#define EXT_MOB1_EXT1_EXT2          0x0000

/* Register 2 PI output */
#define PI1_ON                      0x0200
#define PI1_OFF                     0x0000
#define PI2_ON                      0x0100
#define PI2_OFF                     0x0000
#define PI3_ON                      0x0080
#define PI3_OFF                     0x0000

/* Register 3 PS control */
#define PS_ON                       0x0200
#define PS12_ON                     0x0100
#define PS34_ON                     0x00C0
#define PS_OFF                      0x0000

/* Register 5 Input pin control */
#define IN_CTRL                     0x0100  //0x0000
#define EDGE_HIGH                   0x0000
#define EDGE_LOW                    0x0080
#define CH5_SERI_WIRE               0x0000
#define CH5_TWO_WIRE                0x0040
#define CH5_EN_IN                   0x0000
#define CH5_IN_IN                   0x0020
#define CH6_SERI_WIRE               0x0000
#define CH6_TWO_WIRE                0x0010
#define CH6_EN_IN                   0x0000
#define CH6_IN_IN                   0x0008
#define CH7_SERI_WIRE               0x0000
#define CH7_TWO_WIRE                0x0004
#define CH7_EN_IN                   0x0000
#define CH7_IN_IN                   0x0002

/* Register 6,7 Channel 6,7 control */
#define OFF_MODE                    0x0000
#define BRAKE_MODE                  0x0100

#define VOLT_320mv                  0x0000      /* Constant current value */
#define VOLT_310mv                  0x0010
#define VOLT_300mv                  0x0020
#define VOLT_290mv                  0x0030
#define VOLT_280mv                  0x0040
#define VOLT_270mv                  0x0050
#define VOLT_260mv                  0x0060
#define VOLT_250mv                  0x0070
#define VOLT_240mv                  0x0080
#define VOLT_230mv                  0x0090
#define VOLT_220mv                  0x00A0
#define VOLT_210mv                  0x00B0
#define VOLT_200mv                  0x00C0
#define VOLT_190mv                  0x00D0
#define VOLT_180mv                  0x00E0
#define VOLT_170mv                  0x00F0

/* Register 7 Channel 7 control */
#define WEAK_EXC_OFF                0x0000
#define WEAK_EXC_ON                 0x0008

#define WEAK_VOLT_2                 0x0000      /* One-half of constant current value */
#define WEAK_VOLT_3                 0x0004      /* One-third of constant current value */
    
/* Register 8 */
#define ALL_RESET                   0x0380

//////////////////////////////////////////////////////////////////////////////////////////////////////

#endif /* __PT5111_DEFINE_H__ */ 

